@@ -1,5 +1,5 @@
 /* $Copyright: $
- * Copyright (c) 1996 - 2022 by Steve Baker (ice@mama.indstate.edu)
+ * Copyright (c) 1996 - 2023 by Steve Baker (ice@mama.indstate.edu)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@
  */
 extern FILE *outfile;
 extern const struct linedraw *linedraw;
+extern char xpattern[PATH_MAX];
 
 struct infofile *infostack = NULL;
 
@@ -44,10 +45,10 @@ struct comment *new_comment(struct pattern *phead, char **line, int lines)
   return com;
 }
 
-struct infofile *new_infofile(char *path)
+struct infofile *new_infofile(char *path, bool checkparents)
 {
   struct stat st;
-  char buf[PATH_MAX];
+  char buf[PATH_MAX], rpath[PATH_MAX];
   struct infofile *inf;
   struct comment *chead = NULL, *cend = NULL, *com;
   struct pattern *phead = NULL, *pend = NULL, *p;
@@ -59,6 +60,16 @@ struct infofile *new_infofile(char *path)
   if (i < 0 || !S_ISREG(st.st_mode)) {
     snprintf(buf, PATH_MAX, "%s/.info", path);
     fp = fopen(buf, "r");
+
+    if (fp == NULL && checkparents) {
+      strcpy(rpath, path);
+      while ((fp == NULL) && (strcmp(rpath, "/") != 0)) {
+	snprintf(buf, PATH_MAX, "%.*s/..", PATH_MAX-4, rpath);
+	if (realpath(buf, rpath) == NULL) break;
+	snprintf(buf, PATH_MAX, "%.*s/.info", PATH_MAX-7, rpath);
+	fp = fopen(buf, "r");
+      }
+    }
   } else fp = fopen(path, "r");
   if (fp == NULL) return NULL;
 
@@ -154,10 +165,15 @@ struct comment *infocheck(char *path, char *name, int top, int isdir)
   if (inf == NULL) return NULL;
 
   for(inf = infostack; inf != NULL; inf = inf->next) {
+    int fpos = sprintf(xpattern, "%s/", inf->path);
+
     for(com = inf->comments; com != NULL; com = com->next) {
       for(p = com->pattern; p != NULL; p = p->next) {
 	if (patmatch(path, p->pattern, isdir) == 1) return com;
 	if (top && patmatch(name, p->pattern, isdir) == 1) return com;
+
+	sprintf(xpattern + fpos, "%s", p->pattern);
+	if (patmatch(path, xpattern, isdir) == 1) return com;
       }
     }
     top = 0;
